/******************************************************************************

* XXX_PING

*

* Prior to FF4, we use XPCOM component to initialize our browser record extension

* at the launch of FF.

*

* However, starting FF4(Gecko2), Firefox decided to diminish the support of XPCOM

* and may eventually stop supporting XPCOM inside extension. Starting FF5, XPCOM 

* components now enforce an exact Module::kVersion match, meaning the only way to 

* support multiple major versions of Firefox is to build multiple XPCOM components, 

* one for each major version. To make it even worse, FF is considering new 18-week

* release schedule, this means we would be slaved for re-compiling/re-distribute our

* XPCOM component.

* 

* Therefore, we switched to use js-ctypes and deprecated XPCOM. js-ctypes was supported

* in Gecko2.0(FF4.0) and is recommended by FF to move forward as an alternative for

* XPCOM.

* 

* We still need to keep XPCOM in order to support <= FF3.*. For >= FF4.0, we will use

* js-ctypes instead. Both share the same DLl(nprndlffbrowserrecordext.dll).

*

******************************************************************************/

var g_bFF4AndAbove = false;

var g_oXPCOMBrowserRecord = null;

var g_hRNBrowserRecordExtLib = null;

var g_fnRNInitBrowserRecordExt = null;

var g_fnRNDispathBrowserEvent = null;

var g_fnRNCloseBrowserRecordExt = null;



// Initializing the Browser Record XPCOM component

function createXPCOMBrowserRecord()

{

    const cid = "@real.com/RealPlayer/BrowserRecord;1";

    var oBrowserRecord = Components.classes[cid].createInstance();



    oBrowserRecord = oBrowserRecord.QueryInterface(Components.interfaces.nsIRPBrowserRecord);



    return oBrowserRecord;

}



function onTabSelected(event)

{

    var BrowserEventType_TabWindowOffset_Y = 2;

    calculatePageOffset();



    if (g_fnRNDispathBrowserEvent)

    {

        g_fnRNDispathBrowserEvent(BrowserEventType_TabWindowOffset_Y, window.innerHeight - browser.contentWindow.innerHeight);    

    }

            

    if (g_oXPCOMBrowserRecord)

    {

        g_oXPCOMBrowserRecord.DispatchBrowserEvent(1, 0);

    }

}



function calculatePageOffset()

{

    // Calculate page's height offset for >= FF4.0 and pass it to our

    // record plugin so that we can position our download button properly

    // when the video scroll up & overlap with the tab/menu bar.

    // 

    // We have to do it from JS extension because our plugin can't access

    // the page DOM when it's inside iFrame.

    if (g_bFF4AndAbove)

    {    

        var browser = gBrowser.getBrowserForTab(gBrowser.selectedTab);

        if (browser && browser.contentWindow.innerHeight < window.innerHeight)

        {        

            if (g_fnRNDispathBrowserEvent)

            {

                g_fnRNDispathBrowserEvent(2, window.innerHeight - browser.contentWindow.innerHeight);    

            }

                

            if (g_oXPCOMBrowserRecord)

            {

                g_oXPCOMBrowserRecord.DispatchBrowserEvent(2, window.innerHeight - browser.contentWindow.innerHeight);

            }    

        }

    }

}



// --------------------------------------------------------------------------------------------

// Chrome (un)loading handler.

// --------------------------------------------------------------------------------------------

function onChromeLoad()

{

    if (g_bFF4AndAbove)

    {

        // FF caps SetTimeout's min val at 1000ms 

        setTimeout("calculatePageOffset()", 1000);    

    }

    else

    {

        if (!g_oXPCOMBrowserRecord)

        {

            g_oXPCOMBrowserRecord = createXPCOMBrowserRecord();

        }

    }



    var container = gBrowser.tabContainer;

    container.addEventListener("TabSelect", onTabSelected, false);

}



function onChromeUnload()

{

    if (g_hRNBrowserRecordExtLib)

    {

        if (g_fnRNCloseBrowserRecordExt)

        {

            g_fnRNCloseBrowserRecordExt();

        }

        

        g_hRNBrowserRecordExtLib.Close();

        

        g_hRNBrowserRecordExtLib = null;

        g_fnRNInitBrowserRecordExt = null;

        g_fnRNDispathBrowserEvent = null;

        g_fnRNCloseBrowserRecordExt = null;                

    }

        

    if (g_oXPCOMBrowserRecord)

    {

        delete g_oXPCOMBrowserRecord;

        g_oXPCOMBrowserRecord = null;

    }



    var container = gBrowser.tabContainer;

    container.removeEventListener("TabSelect", onTabSelected, false);

}



var appInfo = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo); 

if (appInfo)

{

    var x = Components.classes["@mozilla.org/xpcom/version-comparator;1"]

                      .getService(Components.interfaces.nsIVersionComparator)

                      .compare(appInfo.version, "4.0");



    // FF >= 4.0

    if (x >= 0)

    {

        g_bFF4AndAbove = true;



        /* XXX_PING

         * DON'T MOVE THE LOGIC BELOW, we want to call g_fnRNInitBrowserRecordExt ASAP because

         * of the async nature of getAddonByID() and we may miss the creation of FF's plugin-container

         * process which is critical to display DTV button

         */ 

        // Initializing the Browser Record C Lib

        if (g_bFF4AndAbove)

        {

            /* Load our addon DLL */   

            Components.utils.import("resource://gre/modules/AddonManager.jsm");

            AddonManager.getAddonByID("{97E22097-9A2F-45b1-8DAF-36AD648C7EF4}", function(addon)

            {

                var addonLocation = addon.getResourceURI("/Components/nprpffbrowserrecordext.dll").QueryInterface(Components.interfaces.nsIFileURL).file.path;



                /* Load JS Ctypes Javascript module */

                Components.utils.import("resource://gre/modules/ctypes.jsm");

                g_hRNBrowserRecordExtLib = ctypes.open(addonLocation);



                if (g_hRNBrowserRecordExtLib)

                {

                    /* Declare the signature of the function we are going to call */

                    g_fnRNInitBrowserRecordExt = g_hRNBrowserRecordExtLib.declare("RNInitBrowserRecordExt",

                                                                                  ctypes.winapi_abi,

                                                                                  ctypes.int32_t);

               

                    if (g_fnRNInitBrowserRecordExt)

                    {

                        g_fnRNInitBrowserRecordExt();

                    }                                                                           

               

                    g_fnRNDispathBrowserEvent = g_hRNBrowserRecordExtLib.declare("RNDispatchBrowserEvent",

                                                                                 ctypes.winapi_abi,

                                                                                 ctypes.int32_t,

                                                                                 ctypes.int32_t,

                                                                                 ctypes.int32_t);

                                                               

                    g_fnRNCloseBrowserRecordExt = g_hRNBrowserRecordExtLib.declare("RNCloseBrowserRecordExt",

                                                                                   ctypes.winapi_abi,

                                                                                   ctypes.int32_t);                                                                           

                }

            });

        }

    }

}



window.addEventListener("load", onChromeLoad, false);

window.addEventListener("unload", onChromeUnload, false);



// ---------- HTML5 VIDEO script injection -------------------

// Place a div containing a html5 plugin around html 5 video tags.



var rpHTML5VideoExtStateEnum = {"Idling":0, "Pending":1, "Scanning":2};

var rpHTML5VideoExtState = rpHTML5VideoExtStateEnum.Pending;



function getVideoSrcFromChildren(video)

{

    var videoSrc = "";

    var children = video.childNodes;



    if (children.length)

    {

        for (var i = 0; i < children.length; i++) 

        {                

            var child = children[i];



            // Look for the first child Source element 

            // with valid src attribute

            if (child.nodeType == 1 &&

                child.src != "")

            {                   

                videoSrc = child.src;

                break;

            }

        }

    }

    

    return videoSrc;

}



function isVideoAttributeChanged(plugin, video)

{

    if (plugin.width != video.clientWidth       ||

        plugin.height != video.clientHeight     ||

        plugin.offsetLeft != video.offsetLeft   ||

        plugin.offsetTop != video.offsetTop)

    {

        return true;

    }



    var pluginSrc = plugin.getAttribute("src");

    var videoSrc = video.currentSrc;

            

    if (pluginSrc != videoSrc)

    {

        return true;

    }

    

    return false;

}



function initRPHTML5VideoPlugin(plugin, index, video)

{

    plugin.id = "RDHTML5Video_Plugin_" + index;

    plugin.type = "application/x-rp-html5videoshim-plugin";

    plugin.width = video.clientWidth;

    plugin.height = video.clientHeight;

    plugin.offsetLeft = video.offsetLeft;

    plugin.offsetTop = video.offsetTop;

    

    var pluginSrc = "";

    

    if (video.currentSrc != "")

    {

        pluginSrc = video.currentSrc;

    }



    if (pluginSrc != "")

    {

        plugin.setAttribute("src", pluginSrc);    

    }   

}



function scanHTML5Videos(document) 

{

    rpHTML5VideoExtState = rpHTML5VideoExtStateEnum.Scanning;

    

    var HTML5Videos = document.getElementsByTagName("video");

    var numOfVideos = HTML5Videos.length;



    for (var i = 0; i < numOfVideos; i++) 

    {                

        var video = HTML5Videos[i];



        var prevSibling = video.previousSibling;

        

        if (prevSibling)

        {

            if (prevSibling.nodeType == 1 && 

                prevSibling.id == "RDHTML5Video")

            {

                var children = prevSibling.childNodes;

                if (children.length)

                {

                    var firstChild = children[0];

                    

                    if (isVideoAttributeChanged(firstChild, video))

                    {

                        // Update parent <div> 

                        prevSibling.style.width = video.clientWidth + "px";

                        prevSibling.style.height = video.clientHeight + "px";

                        initRPHTML5VideoPlugin(firstChild, i, video);

                    }

                }

                continue;

            }

        }

            

        var _div = document.createElement('div');

        _div.setAttribute("id", "RDHTML5Video");

        _div.setAttribute("style", "position: absolute; z-index: 100");

        _div.style.width = video.clientWidth + "px";

        _div.style.height = video.clientHeight + "px";



        var _rpHTML5VideoPlugin = document.createElement('object')        

        initRPHTML5VideoPlugin(_rpHTML5VideoPlugin, i, video);                

         

        _div.appendChild(_rpHTML5VideoPlugin);



        video.parentNode.insertBefore(_div, video);

    }



    rpHTML5VideoExtState = rpHTML5VideoExtStateEnum.Idling;

}



function OnDocumentLoaded(event)

{

    var doc = event.originalTarget;

    if (doc instanceof HTMLDocument) 

    {  

        // is this an inner frame?  

        if (doc.defaultView.frameElement) 

        {  

            // Frame within a tab was loaded.  

            // Find the root document:  

            while (doc.defaultView.frameElement) 

            {  

                doc = doc.defaultView.frameElement.ownerDocument;  

            }  

        }  

    }    

    

    scanHTML5Videos(doc);

}



gBrowser.addEventListener("load", OnDocumentLoaded, true); //fires when browser starts and for each page.



